// --------------------------------------------------------------------
// Scripted behaviour: Bouncing
//
// This file contains a simple script that causes the associated object
// to bounce around the canvas.
// --------------------------------------------------------------------
// Events: Bouncing.OnEnter, Bouncing.OnUpdate
// --------------------------------------------------------------------

// Constants
const float BOUNCE_SPEED = 150.0f;
const float BOUNCE_MARGIN = 0.0f;
const float BOUNCE_GRAVITY = 0.15f;

// Called at the start of the page
event Bouncing.OnEnter
{
	// Set initial velocity,rotation
	self.$velocity = Vector2(randf()*6.0f - 3.0f,randf()*2.0f - 1.0f);	
	self.$rotation = self.$velocity.AsFloat() * BOUNCE_SPEED * 0.02f;	
	
	// Adjust pivot point
	self.pivot = self.size * 0.5f;
	self.position += self.size * 0.5f * self.scale;
}


// Bouncing behaviour
event Bouncing.OnUpdate
{
	// Manually call Bouncing.OnEnter (once only)
	if (_VAR_EXISTS(self,"velocity") == false)
		__Bouncing_OnEnter(self);
	
	// Get velocity, current position
	Vector2 velocity = self.$velocity.AsVector2();
	Vector2 currPos = self.position - (self.pivot * self.scale);

	// Apply gravity
	velocity.y += BOUNCE_GRAVITY * elapsed * BOUNCE_SPEED;

	// Check for horizontal bounce
	if ((velocity.x < 0 && currPos.x < 0.0f) || 
	(velocity.x > 0 && currPos.x + self.size.x * self.scale * (1.0f-BOUNCE_MARGIN) > 800)		)
	{
		velocity.x *= -1.0f;		// Invert horizontal velocity
		self.$rotation *= -1.0f;	// Flip rotation speed
	}

	// Check for vertical bounce
	if (velocity.y > 0 && currPos.y + self.size.y * self.scale * (1.0f-BOUNCE_MARGIN) > 600)
	{
		velocity.y *= -0.9f;		// Invert vertical velocity, lose energy
		if (abs(velocity.y) < 0.5f)	// If the bounce is small enough..
			velocity.y = 10.0f;		// ..Pop!
	}
	
	// Update new position, rotation and velocity
	self.position = self.position + velocity * elapsed * BOUNCE_SPEED;
	self.rotation += self.$rotation.AsFloat() * elapsed;
	self.$velocity = velocity;
}